// test-Pulsar
//
// Copyright © 1998 Arcane Dragon Software
//                  Chris Herborth (chrish@kagi.com)
//
// A simple test of the ChPulsar object; we'll create a window with two
// views... one view will be updated by Pulse(), the other will be
// updated by the ChPulsar.
//
// You shouldn't see much of a difference, unless the app is totally
// flooded with mouse/keyboard/redraw messsages.  In that case, Pulse()
// won't be called, but the ChPulsar should still send its message (unless
// the view's message queue is full).
//
// The views keep track of the number of counter updates; every 5th one
// they should update themselves.  If you could have multiple pulse rates
// per window, this wouldn't be necessary.  Or, you could create another
// ChPulsar that sent an "update yourself" message every 5 seconds...

#include "Ch/ChPulsar.h"

#include <app/Application.h>
#include <interface/View.h>
#include <interface/Window.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <kernel/OS.h>

static char *_rcs_id = "$Id: test-Pulsar.cpp,v 1.3 1998/04/14 00:49:43 chrish Exp $";

// .25 seconds, in microseconds
const bigtime_t QUARTER_SECOND = ( 1 * 1000 * 1000 / 4 );

// Our custom message
const uint32 PULSAR_MSG = 'puls';

// ----------------------------------------------------------------------
// The Pulse()-handling view.
class PulseView : public BView
{
public:
	PulseView( BRect &frame );
	virtual ~PulseView( void ) { ; }

	virtual void AttachedToWindow( void );	
	virtual void Draw( BRect update );
	virtual void Pulse( void );

private:
	int32 _pulse_count;
	char _buff[256];
};

PulseView::PulseView( BRect &frame )
	: BView( frame, "Pulse view", B_FOLLOW_ALL_SIDES,
			 B_WILL_DRAW | B_PULSE_NEEDED | B_FULL_UPDATE_ON_RESIZE )
{
	_pulse_count = 0;
	_buff[0] = '\0';
}

void PulseView::AttachedToWindow( void )
{
	if( Window()->Lock() ) {
		Window()->SetPulseRate( QUARTER_SECOND );
		Window()->Unlock();
	}
}

void PulseView::Draw( BRect update )
{
#pragma unused (update)

	sprintf( _buff, "count = %d", _pulse_count );
	if( Window()->Lock() ) {
		MovePenTo( 5.0, 15.0 );
		DrawString( _buff );
		Window()->Unlock();
	}

	uint32 now = real_time_clock();
	sprintf( _buff, "time = %s", ctime( (long *)&now ) );
	_buff[strlen(_buff)-1] = '\0';
	if( Window()->Lock() ) {
		MovePenTo( 5.0, 30.0 );
		DrawString( _buff );
		Window()->Unlock();
	}
}

void PulseView::Pulse( void )
{
	_pulse_count++;

	if( _pulse_count % 5 == 0 ) {
		FillRect( Bounds(), B_SOLID_LOW );
		Draw( Bounds() );
		Sync();
	}
}

// ----------------------------------------------------------------------
// The ChPulsar-handling view.
class TestView: public BView
{
public:
	TestView( BRect &frame );
	virtual ~TestView( void ) { ; }

	virtual void Draw( BRect update );
	virtual void MessageReceived( BMessage *msg );

private:
	int32 _pulse_count;
	char _buff[256];

	time_t _pulsar_time;
	double _temp;
};

TestView::TestView( BRect &frame )
	: BView( frame, "Pulsar view", B_FOLLOW_ALL_SIDES,
			 B_WILL_DRAW | B_FULL_UPDATE_ON_RESIZE )
{
	_pulse_count = 0;
	_buff[0] = '\0';

	_pulsar_time = 0;
	_temp = 0.0;
}

void TestView::Draw( BRect update )
{
#pragma unused (update)

	sprintf( _buff, "count = %d", _pulse_count );
	if( Window()->Lock() ) {
		MovePenTo( 5.0, 15.0 );
		DrawString( _buff );
		Window()->Unlock();
	}

	uint32 now = real_time_clock();
	sprintf( _buff, "time = %s", ctime( (long *)&now ) );
	_buff[strlen(_buff)-1] = '\0';
	if( Window()->Lock() ) {
		MovePenTo( 5.0, 30.0 );
		DrawString( _buff );
		Window()->Unlock();
	}

	sprintf( _buff, "pulsar time = %s", ctime( &_pulsar_time ) );
	_buff[strlen(_buff)-1] = '\0';
	if( Window()->Lock() ) {
		MovePenTo( 5.0, 45.0 );
		DrawString( _buff );
		Window()->Unlock();
	}

	sprintf( _buff, "motherboard temp. = %lf", _temp );
	if( Window()->Lock() ) {
		MovePenTo( 5.0, 60.0 );
		DrawString( _buff );
		Window()->Unlock();
	}
}

void TestView::MessageReceived( BMessage *msg )
{
	switch( msg->what ) {
	case PULSAR_MSG:
		_pulse_count++;
		_pulsar_time = msg->FindInt32( "pulsar_time" );
		_temp = msg->FindDouble( "pulsar_fire" );

		if( _pulse_count % 5 == 0 ) {
			FillRect( Bounds(), B_SOLID_LOW );
			Draw( Bounds() );
			Sync();
		}

		// We handled this message, so return.
		return;
		break;

	default:
		break;
	}

	// Pass it on...	
	BView::MessageReceived( msg );
}

// ----------------------------------------------------------------------
// test application
class TestApp: public BApplication
{
public:
	TestApp( void );
	virtual ~TestApp( void ) { ; }

private:
	BWindow *_wind;
	ChPulsar *_pulsar;
};

TestApp::TestApp( void )
	: BApplication( "application/x-vnd.ads-pulsartest" )
{
	// Set up our window, with two views inside.
	_wind = new BWindow( BRect( 50, 50, 600, 300 ), "Pulsar test",
						 B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
						 0 );
	if( _wind ) {
		BView *view = new PulseView( BRect( 50, 50, 300, 300 ) );
		if( view ) {
			_wind->AddChild( view );
		}

		view = new TestView( BRect( 301, 50, 600, 300 ) );
		if( view ) {
			_wind->AddChild( view );
		}

		_wind->Show();
	}

	// Here's the ChPulsar...
	_pulsar = new ChPulsar( QUARTER_SECOND, new BMessage( PULSAR_MSG ),
							"test Pulsar", _wind->FindView( "Pulsar view" ),
							B_NORMAL_PRIORITY );
	if( _pulsar->IsValid() ) {
		_pulsar->Start();
	} else {
		printf( "_pulsar isn't valid!\n" );
	}
}

// ----------------------------------------------------------------------
// main()
int main( void )
{
	printf( "ChPulsar test\n" );
	printf( "%s\n", _rcs_id );
	printf( "\n" );

	TestApp the_app;
	the_app.Run();

	return 0;
}
